--[[
SimpleHalo
Author: Michael Joseph Murray aka Lyte of Lothar(US)
$Revision: 4 $
$Date: 2012-10-25 18:37:14 +0000 (Thu, 25 Oct 2012) $
Project Version: r250
contact: codemaster2010 AT gmail DOT com

Copyright (c) 2012 Michael J. Murray aka Lyte of Lothar(US)
All rights reserved unless otherwise explicitly stated.
]]

if select(2, UnitClass("player")) ~= "PRIEST" then return end

SimpleHalo = LibStub("AceAddon-3.0"):NewAddon("SimpleHalo", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("SimpleHalo")
local LSM = LibStub("LibSharedMedia-3.0")
local LRC = LibStub("LibRangeCheck-2.0")

function SimpleHalo:OnInitialize()
	local defaults = {
		profile = {
			position = {},
			locked = false,
			scale = 1,
			unit = "target",
			customUnit = "none",
		},
	}
	
	self.db = LibStub("AceDB-3.0"):New("SimpleHaloDB", defaults, "Default")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Refresh")
	self.db.RegisterCallback(self, "OnProfileChanged", "Refresh")
	
	local opts = {
		type = 'group',
		args = {
			display = {
				type = 'group',
				name = "Display",
				args = {
					configMode = {
						type = 'toggle',
						name = L["Configuration Mode"],
						desc = L["Force the indicator to display for configuration purposes."],
						get = function() return configMode end,
						set = function()
							configMode = not configMode
							
							if configMode then
								self.indicator:Show()
							else
								self.indicator:Hide()
							end
						end,
						order = 1,
					},
					locked = {
						type = 'toggle',
						name = L["Lock Frame"],
						desc = L["Lock the frame to prevent movement."],
						get = function() return self.db.profile.locked end,
						set = function() self.db.profile.locked = not self.db.profile.locked end,
						order = 2,
					},
					scale = {
						type = 'range',
						name = L["Scale"],
						desc = L["Scale the frame to make it smaller or larger."],
						get = function() return self.db.profile.scale end,
						set = function(_, v)
							self.db.profile.scale = v
							self.indicator:SetHeight(50*v)
							self.indicator:SetWidth(100*v)
							
							self.indicator.icon:SetHeight(25*v)
							self.indicator.icon:SetWidth(25*v)

							self.indicator.text:SetHeight(25*v)
							self.indicator.text:SetWidth(25*v)
						end,
						min = 0.5,
						max = 2.5,
						step = 0.01,
						order = 3,
					},
					unit = {
						type = 'select',
						name = L["Unit to Watch"],
						desc = L["Choose the unit to check for proper Halo range. Target is recommended."],
						get = function() return self.db.profile.unit end,
						set = function(_, v)
							self.db.profile.unit = v
							if v == "custom" then
								self.indicator.unit = self.db.profile.customUnit
							else
								self.indicator.unit = v
							end
						end,
						values = {
							target = L["Target"],
							targettarget = L["Target's target"],
							boss1 = L["Boss Frame One"],
							boss2 = L["Boss Frame Two"],
							boss3 = L["Boss Frame Three"],
							boss4 = L["Boss Frame Four"],
							boss5 = L["Boss Frame Five"],
							focus = L["Focus"],
							focustarget = L["Focus' Target"],
							custom = L["Custom Unit"],
						},
						order = 4,
					},
					unitText = {
						type = 'description',
						order = 5,
						name = L["customUnit_helpText"],
						hidden = function()
							if self.db.profile.unit ~= 'custom' then return true else return false end
						end,
					},
					customUnit = {
						type = 'input',
						name = L["Custom Unit"],
						desc = L["Specify a custom UnitID to check for Halo range. For advanced users only!"],
						get = function() return self.db.profile.customUnit end,
						set = function(_, v)
							self.db.profile.customUnit = v
							self.indicator.unit = v
						end,
						hidden = function()
							if self.db.profile.unit ~= 'custom' then return true else return false end
						end,
						order = 6,
					},
				},
			},
		},
	}
	opts.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SimpleHalo", opts)
	LibStub("AceConfigDialog-3.0"):SetDefaultSize("SimpleHalo", 500, 300)
	
	self:RegisterChatCommand("halo", "OpenConfig", true, true)
	self:RegisterChatCommand("simplehalo", "OpenConfig")
	
	self.indicator = self:CreateIndicator()
end

function SimpleHalo:OnEnable()
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "LeaveCombat")
	self:RegisterEvent("PLAYER_REGEN_DISABLED", "EnterCombat")
end

function SimpleHalo:Reset()
	self.indicator:SetHeight(50)
	self.indicator:SetWidth(100)
	
	self.indicator.icon:SetHeight(25)
	self.indicator.icon:SetWidth(25)

	self.indicator.text:SetHeight(25)
	self.indicator.text:SetWidth(25)
	
	self.indicator:ClearAllPoints()
	self.indicator:SetPoint("CENTER", UIParent, "CENTER")
	
	self.indicator:EnableMouse(true)
end

function SimpleHalo:Refresh()
	local db = self.db.profile
	
	self.indicator:SetHeight(50*db.scale)
	self.indicator:SetWidth(100*db.scale)
	
	self.indicator.icon:SetHeight(25*db.scale)
	self.indicator.icon:SetWidth(25*db.scale)

	self.indicator.text:SetHeight(25*db.scale)
	self.indicator.text:SetWidth(25*db.scale)
	
	self.indicator:ClearAllPoints()
	if db.position.x then
		self.inticator:SetPoint(db.position.point, UIParent, db.position.anchor, db.position.x, db.position.y)
	else
		self.indicator:SetPoint("CENTER", UIParent, "CENTER")
	end
	
	if db.locked then
		self.indicator:EnableMouse(false)
	else
		self.indicator:EnableMouse(true)
	end
end

function SimpleHalo:OpenConfig()
	LibStub("AceConfigDialog-3.0"):Open("SimpleHalo")
end

function SimpleHalo:EnterCombat()
	--show the frame and start range polling
	self.indicator:Show()
end

function SimpleHalo:LeaveCombat()
	--hide the frame to stop range polling
	self.indicator:Hide()
end

local lastUpdate = 0
local function updateRange(frame, elapsed)
	lastUpdate = lastUpdate + elapsed
	if lastUpdate < 0.125 then return end
	
	lastUpdate = 0
	
	local minRange, maxRange = LRC:GetRange(frame.unit)
	
	--This is the same logic used by HaloPro, which the Theorycrafters love
	if (not minRange) or (not maxRange) then
		--no range information
		frame.icon:SetVertexColor(1,1,1)
		frame.text:SetText("")
	elseif maxRange <= 15 then
		--within 15 yards of target
		--too close!
		frame.icon:SetVertexColor(1,0,0)
		frame.text:SetText("-")
	elseif minRange >= 15 and maxRange <= 20 then
		--between 15 and 20 yards of target
		--slightly too close
		frame.icon:SetVertexColor(1,1,0)
		frame.text:SetText("-")
	elseif minRange >= 20 and maxRange <= 25 then
		--sweet spot
		frame.icon:SetVertexColor(0,1,0)
		frame.text:SetText("")
	elseif minRange >= 25 and maxRange <=30 then
		--between 25 and 30 yards of target
		--slightly too far
		frame.icon:SetVertexColor(1,1,0)
		frame.text:SetText("+")
	elseif minRange > 30 then
		--more than 30 yards
		--too far!
		frame.icon:SetVertexColor(1,0,0)
		frame.text:SetText("+")
	end
end

function SimpleHalo:CreateIndicator()
	local icon = [[Interface\AddOns\SimpleHalo\icon.tga]]
	local db = self.db.profile
	local f = CreateFrame("FRAME", nil, UIParent)
	
	--size and positon
	f:SetWidth(100*db.scale)
	f:SetHeight(50*db.scale)
	
	if self.db.profile.position.x and self.db.profile.position.y and self.db.profile.position.anchor and self.db.profile.position.point then
		f:ClearAllPoints()
		f:SetPoint(self.db.profile.position.point, UIParent, self.db.profile.position.anchor, self.db.profile.position.x, self.db.profile.position.y)
	else
		f:SetPoint("CENTER", UIParent, "CENTER")
	end
	
	--style the frame
	f:SetBackdrop({
		bgFile = [[Interface\Tooltips\UI-Tooltip-Background]],
		edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]],
		tile = false, tileSize = 16, edgeSize = 16,
		insets = { left = 5, right = 5, top = 5, bottom = 5 }
	})
	f:SetBackdropColor(0.65, 0.65, 0.65, 0.65)
	
	f.icon = f:CreateTexture()
	f.icon:SetTexture(icon)
	f.icon:SetHeight(25*db.scale)
	f.icon:SetWidth(25*db.scale)
	f.icon:ClearAllPoints()
	f.icon:SetPoint("LEFT", f, "LEFT", 5, 0)
	
	f.text = f:CreateFontString()
	f.text:SetHeight(25*db.scale)
	f.text:SetWidth(25*db.scale)
	f.text:SetFont([[Fonts\FRIZQT__.TTF]], 24)
	f.text:SetTextColor(1, 1, 1, 1)
	f.text:SetJustifyH("LEFT")
	f.text:SetJustifyV("MIDDLE")
	f.text:ClearAllPoints()
	f.text:SetPoint("RIGHT", f, "RIGHT", -5, 0)
	
	--setup dragging
	f:SetClampedToScreen(true)
	f:SetMovable(true)
	f:RegisterForDrag("LeftButton")
	if self.db.profile.locked then
		f:EnableMouse(false)
	else
		f:EnableMouse(true)
	end
	
	f:SetScript("OnDragStart", function() f:StartMoving() end )
	f:SetScript("OnDragStop", function ()
		f:StopMovingOrSizing()
		local point, _, anchor, x, y = f:GetPoint()
		SimpleHalo.db.profile.position.x = floor(x)
		SimpleHalo.db.profile.position.y = floor(y)
		SimpleHalo.db.profile.position.anchor = anchor
		SimpleHalo.db.profile.position.point = point
	end)
	
	if self.db.profile.unit ~= "custom" then
		f.unit = self.db.profile.unit
	else
		f.unit = self.db.profile.customUnit
	end
	
	--set our update script
	f:SetScript("OnUpdate", updateRange)
	
	--hide and return
	f:Hide()
	return f
end
